@@ -5,7 +5,8 @@
  *
  * Authored By Matthew Allum  <mallum@openedhand.com>
  *
- * Copyright (C) 2006 OpenedHand
+ * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd
+ * Copyright (C) 2009, 2010 Intel Corp
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -93,21 +94,26 @@ typedef void (*ClutterCallback) (ClutterActor *actor,
 
 /**
  * ClutterActorFlags:
- * @CLUTTER_ACTOR_MAPPED: the actor will be painted (is visible, and inside a toplevel, and all parents visible)
+ * @CLUTTER_ACTOR_MAPPED: the actor will be painted (is visible, and inside
+ *   a toplevel, and all parents visible)
  * @CLUTTER_ACTOR_REALIZED: the resources associated to the actor have been
  *   allocated
  * @CLUTTER_ACTOR_REACTIVE: the actor 'reacts' to mouse events emmitting event
  *   signals
  * @CLUTTER_ACTOR_VISIBLE: the actor has been shown by the application program
+ * @CLUTTER_ACTOR_NO_LAYOUT: the actor provides an explicit layout management
+ *   policy for its children; this flag will prevent Clutter from automatic
+ *   queueing of relayout and will defer all layouting to the actor itself
  *
  * Flags used to signal the state of an actor.
  */
 typedef enum
 {
-  CLUTTER_ACTOR_MAPPED   = 1 << 1,
-  CLUTTER_ACTOR_REALIZED = 1 << 2,
-  CLUTTER_ACTOR_REACTIVE = 1 << 3,
-  CLUTTER_ACTOR_VISIBLE  = 1 << 4
+  CLUTTER_ACTOR_MAPPED    = 1 << 1,
+  CLUTTER_ACTOR_REALIZED  = 1 << 2,
+  CLUTTER_ACTOR_REACTIVE  = 1 << 3,
+  CLUTTER_ACTOR_VISIBLE   = 1 << 4,
+  CLUTTER_ACTOR_NO_LAYOUT = 1 << 5
 } ClutterActorFlags;
 
 /**
@@ -128,6 +134,25 @@ typedef enum
   CLUTTER_ABSOLUTE_ORIGIN_CHANGED = 1 << 1
 } ClutterAllocationFlags;
 
+/**
+ * ClutterRedrawFlags:
+ * @CLUTTER_REDRAW_CLIPPED_TO_BOX: Tells clutter the redraw is clipped
+ *   to a given clip box in actor coordinates.
+ * @CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION: Tells clutter the maximum
+ *   extents of what needs to be redrawn lies within the actors
+ *   current allocation.
+ *
+ * Flags passed to the clutter_actor_queue_redraw_with_clip ()
+ * function
+ *
+ * Since: 1.2
+ */
+typedef enum
+{
+  CLUTTER_REDRAW_CLIPPED_TO_BOX         = 0,
+  CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION  = 1 << 1
+} ClutterRedrawFlags;
+
 /**
  * ClutterActor:
  * @flags: #ClutterActorFlags
@@ -178,7 +203,11 @@ struct _ClutterActor
  *   and natural heights of an actor for a given width; it is used by
  *   clutter_actor_get_preferred_height()
  * @allocate: virtual function, used when settings the coordinates of an
- *   actor; it is used by clutter_actor_allocate()
+ *   actor; it is used by clutter_actor_allocate(); it must chain up to
+ *   the parent's implementation
+ * @apply_transform: virtual function, used when applying the transformations
+ *   to an actor before painting it or when transforming coordinates or
+ *   the allocation; it must chain up to the parent's implementation
  * @parent_set: signal class handler for the #ClutterActor::parent-set
  * @destroy: signal class handler for #ClutterActor::destroy
  * @pick: virtual function, used to draw an outline of the actor with
@@ -198,6 +227,7 @@ struct _ClutterActor
  * @captured_event: signal class closure for #ClutterActor::captured-event
  * @key_focus_in: signal class closure for #ClutterActor::focus-in
  * @key_focus_out: signal class closure for #ClutterActor::focus-out
+ * @queue_relayout: class handler for #ClutterActor::queue-relayout
  *
  * Base class for actors.
  */
@@ -238,6 +268,11 @@ struct _ClutterActorClass
   void (* allocate)             (ClutterActor           *actor,
                                  const ClutterActorBox  *box,
                                  ClutterAllocationFlags  flags);
+
+  /* transformations */
+  void (* apply_transform)      (ClutterActor           *actor,
+                                 CoglMatrix             *matrix);
+
   /* event signals */
   gboolean (* event)                (ClutterActor         *actor,
                                      ClutterEvent         *event);
@@ -262,9 +297,11 @@ struct _ClutterActorClass
   void     (* key_focus_in)         (ClutterActor         *actor);
   void     (* key_focus_out)        (ClutterActor         *actor);
 
+  void     (* queue_relayout)       (ClutterActor         *actor);
+
   /*< private >*/
   /* padding for future expansion */
-  gpointer _padding_dummy[32];
+  gpointer _padding_dummy[31];
 };
 
 GType                 clutter_actor_get_type                  (void) G_GNUC_CONST;
@@ -284,13 +321,15 @@ void                  clutter_actor_unrealize                 (ClutterActor
 void                  clutter_actor_map                       (ClutterActor          *self);
 void                  clutter_actor_unmap                     (ClutterActor          *self);
 void                  clutter_actor_paint                     (ClutterActor          *self);
-void                  clutter_actor_pick                      (ClutterActor          *self,
-                                                               const ClutterColor    *color);
 void                  clutter_actor_queue_redraw              (ClutterActor          *self);
+
 void                  clutter_actor_queue_relayout            (ClutterActor          *self);
 void                  clutter_actor_destroy                   (ClutterActor          *self);
 
 /* size negotiation */
+void                  clutter_actor_set_request_mode          (ClutterActor          *self,
+                                                               ClutterRequestMode     mode);
+ClutterRequestMode    clutter_actor_get_request_mode          (ClutterActor          *self);
 void                  clutter_actor_get_preferred_width       (ClutterActor          *self,
                                                                gfloat                 for_height,
                                                                gfloat                *min_width_p,
@@ -315,19 +354,13 @@ void                  clutter_actor_allocate_available_size   (ClutterActor
                                                                gfloat                 available_width,
                                                                gfloat                 available_height,
                                                                ClutterAllocationFlags flags);
-void                  clutter_actor_get_allocation_coords     (ClutterActor          *self,
-                                                               gint                  *x_1,
-                                                               gint                  *y_1,
-                                                               gint                  *x_2,
-                                                               gint                  *y_2);
 void                  clutter_actor_get_allocation_box        (ClutterActor          *self,
                                                                ClutterActorBox       *box);
 void                  clutter_actor_get_allocation_geometry   (ClutterActor          *self,
                                                                ClutterGeometry       *geom);
 void                  clutter_actor_get_allocation_vertices   (ClutterActor          *self,
 							       ClutterActor          *ancestor,
-                                                               ClutterVertex          verts[4]);
-
+                                                               ClutterVertex          verts[]);
 void                  clutter_actor_set_geometry              (ClutterActor          *self,
                                                                const ClutterGeometry *geometry);
 void                  clutter_actor_get_geometry              (ClutterActor          *self,
@@ -499,7 +532,7 @@ gboolean clutter_actor_is_scaled                      (ClutterActor   *self);
 gboolean clutter_actor_should_pick_paint              (ClutterActor   *self);
 
 void clutter_actor_get_abs_allocation_vertices        (ClutterActor        *self,
-                                                       ClutterVertex        verts[4]);
+                                                       ClutterVertex        verts[]);
 
 void clutter_actor_apply_transform_to_point           (ClutterActor        *self,
                                                        const ClutterVertex *point,
@@ -516,6 +549,19 @@ PangoContext *clutter_actor_create_pango_context      (ClutterActor        *self
 PangoLayout * clutter_actor_create_pango_layout       (ClutterActor        *self,
                                                        const gchar         *text);
 
+void clutter_actor_get_transformation_matrix          (ClutterActor        *self,
+                                                       CoglMatrix          *matrix);
+
+gboolean clutter_actor_is_in_clone_paint              (ClutterActor        *self);
+gboolean clutter_actor_has_pointer                    (ClutterActor        *self);
+
+void                 clutter_actor_set_text_direction (ClutterActor         *self,
+                                                       ClutterTextDirection  text_dir);
+ClutterTextDirection clutter_actor_get_text_direction (ClutterActor         *self);
+
+void                 clutter_actor_push_internal      (ClutterActor         *self);
+void                 clutter_actor_pop_internal       (ClutterActor         *self);
+
 G_END_DECLS
 
-#endif /* _HAVE_CLUTTER_ACTOR_H */
+#endif /* __CLUTTER_ACTOR_H__ */
