@@ -12,6 +12,24 @@ G_BEGIN_DECLS
 GType clutter_actor_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_ACTOR_FLAGS (clutter_actor_flags_get_type())
 
+GType clutter_allocation_flags_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_ALLOCATION_FLAGS (clutter_allocation_flags_get_type())
+
+GType clutter_redraw_flags_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_REDRAW_FLAGS (clutter_redraw_flags_get_type())
+
+/* enumerations from "./clutter-animator.h" */
+GType clutter_interpolation_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_INTERPOLATION (clutter_interpolation_get_type())
+
+/* enumerations from "./clutter-bin-layout.h" */
+GType clutter_bin_alignment_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_BIN_ALIGNMENT (clutter_bin_alignment_get_type())
+
+/* enumerations from "./clutter-box-layout.h" */
+GType clutter_box_alignment_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_BOX_ALIGNMENT (clutter_box_alignment_get_type())
+
 /* enumerations from "./clutter-event.h" */
 GType clutter_modifier_type_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_MODIFIER_TYPE (clutter_modifier_type_get_type())
@@ -32,6 +50,14 @@ GType clutter_stage_state_get_type (void) G_GNUC_CONST;
 GType clutter_feature_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_FEATURE_FLAGS (clutter_feature_flags_get_type())
 
+/* enumerations from "./clutter-flow-layout.h" */
+GType clutter_flow_orientation_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_FLOW_ORIENTATION (clutter_flow_orientation_get_type())
+
+/* enumerations from "./clutter-input-device.h" */
+GType clutter_input_device_type_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_INPUT_DEVICE_TYPE (clutter_input_device_type_get_type())
+
 /* enumerations from "./clutter-main.h" */
 GType clutter_init_error_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_INIT_ERROR (clutter_init_error_get_type())
@@ -48,6 +74,10 @@ GType clutter_script_error_get_type (void) G_GNUC_CONST;
 GType clutter_shader_error_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_SHADER_ERROR (clutter_shader_error_get_type())
 
+/* enumerations from "./clutter-stage.h" */
+GType clutter_pick_mode_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_PICK_MODE (clutter_pick_mode_get_type())
+
 /* enumerations from "./clutter-texture.h" */
 GType clutter_texture_error_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_TEXTURE_ERROR (clutter_texture_error_get_type())
@@ -81,6 +111,13 @@ GType clutter_animation_mode_get_type (void) G_GNUC_CONST;
 GType clutter_font_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_FONT_FLAGS (clutter_font_flags_get_type())
 
+GType clutter_text_direction_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_TEXT_DIRECTION (clutter_text_direction_get_type())
+
+/* enumerations from "./clutter-units.h" */
+GType clutter_unit_type_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_UNIT_TYPE (clutter_unit_type_get_type())
+
 G_END_DECLS
 
 #endif /* !__CLUTTER_ENUM_TYPES_H__ */
