@@ -20,9 +20,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * NB: Inspiration for column storage taken from GtkListStore
  */
@@ -110,10 +110,34 @@
  * }
  * </programlisting></informalexample>
  *
- * #ClutterModel is an abstract class. Clutter provides a default model
- * implementation called #ClutterModelDefault which has been optimised
+ * #ClutterModel is an abstract class. Clutter provides a list model
+ * implementation called #ClutterListModel which has been optimised
  * for insertion and look up in sorted lists.
  *
+ * <refsect2 id="ClutterModel-script">
+ *   <title>ClutterModel custom properties for #ClutterScript</title>
+ *   <para>#ClutterModel defines a custom property "columns" for #ClutterScript
+ *   which allows defining the column names and types.</para>
+ *   <example id="ClutterModel-script-column-example">
+ *     <title>Example of the "columns" custom property</title>
+ *     <para>The definition below will create a #ClutterListModel with three
+ *     columns: the first one with name "Name" and containing strings; the
+ *     second one with name "Score" and containing integers; the third one with
+ *     name "Icon" and containing #ClutterTexture<!-- -->s.</para>
+ *     <programlisting>
+ *  {
+ *    "type" : "ClutterListModel",
+ *    "id" : "teams-model",
+ *    "columns" : [
+ *      [ "Name", "gchararray" ],
+ *      [ "Score", "gint" ],
+ *      [ "Icon", "ClutterTexture" ]
+ *    ]
+ *  }
+ *     </programlisting>
+ *   </example>
+ * </refsect2>
+ *
  * #ClutterModel is available since Clutter 0.6
  */
 
@@ -132,9 +156,21 @@
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-debug.h"
+#include "clutter-scriptable.h"
 
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
 
-G_DEFINE_ABSTRACT_TYPE (ClutterModel, clutter_model, G_TYPE_OBJECT);
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE
+        (ClutterModel, clutter_model, G_TYPE_OBJECT,
+         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                clutter_scriptable_iface_init));
+
+enum
+{
+  PROP_0,
+
+  PROP_FILTER_SET
+};
 
 enum
 {
@@ -157,6 +193,15 @@ struct _ClutterModelPrivate
 {
   GType                  *column_types;
   gchar                 **column_names;
+
+  /* we use an integer here because we want to be able to use -1 as a
+   * guard value, to allow calling set_names() and set_types() from
+   * sub-classes of ClutterModel. see bug:
+   *
+   *   http://bugzilla.openedhand.com/show_bug.cgi?id=2032
+   *
+   * for a reference.
+   */
   gint                    n_columns; 
 
   ClutterModelFilterFunc  filter_func;
@@ -199,29 +244,12 @@ clutter_model_real_get_column_name (ClutterModel *model,
 static guint
 clutter_model_real_get_n_columns (ClutterModel *model)
 {
-  return model->priv->n_columns;
-}
+  ClutterModelPrivate *priv = model->priv;
 
-static guint
-clutter_model_real_get_n_rows (ClutterModel *model)
-{
-  ClutterModelIter *iter;
-  guint n_rows = 0;
-
-  iter = clutter_model_get_first_iter (model);
-  if (!iter)
+  if (priv->n_columns < 0)
     return 0;
 
-  while (!clutter_model_iter_is_last (iter))
-    {
-      n_rows += 1;
-
-      clutter_model_iter_next (iter);
-    }
-
-  g_object_unref (iter);
-
-  return n_rows;
+  return priv->n_columns;
 }
 
 static void 
@@ -238,22 +266,47 @@ clutter_model_finalize (GObject *object)
 
   g_free (priv->column_types);
 
-  /* the column_names vector might have holes in it, so we need to
-   * use the columns number to clear up everything
-   */
-  for (i = 0; i < priv->n_columns; i++)
-    g_free (priv->column_names[i]);
+  if (priv->column_names != NULL)
+    {
+      /* the column_names vector might have holes in it, so we need
+       * to use the columns number to clear up everything
+       */
+      for (i = 0; i < priv->n_columns; i++)
+        g_free (priv->column_names[i]);
 
-  g_free (priv->column_names);
+      g_free (priv->column_names);
+    }
 
   G_OBJECT_CLASS (clutter_model_parent_class)->finalize (object);
 }
 
+static void
+clutter_model_get_property (GObject    *gobject,
+                            guint       prop_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
+{
+  ClutterModelPrivate *priv = CLUTTER_MODEL (gobject)->priv;
+
+  switch (prop_id)
+    {
+    case PROP_FILTER_SET:
+      g_value_set_boolean (value, priv->filter_func != NULL);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+      break;
+    }
+}
+
 static void
 clutter_model_class_init (ClutterModelClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GParamSpec *pspec;
 
+  gobject_class->get_property = clutter_model_get_property;
   gobject_class->finalize = clutter_model_finalize;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterModelPrivate));
@@ -261,7 +314,23 @@ clutter_model_class_init (ClutterModelClass *klass)
   klass->get_column_name  = clutter_model_real_get_column_name;
   klass->get_column_type  = clutter_model_real_get_column_type;
   klass->get_n_columns    = clutter_model_real_get_n_columns;
-  klass->get_n_rows       = clutter_model_real_get_n_rows;
+
+  /**
+   * ClutterModel:filter-set:
+   *
+   * Whether the #ClutterModel has a filter set
+   *
+   * This property is set to %TRUE if a filter function has been
+   * set using clutter_model_set_filter()
+   *
+   * Since: 1.0
+   */
+  pspec = g_param_spec_boolean ("filter-set",
+                                "Filter Set",
+                                "Whether the model has a filter",
+                                FALSE,
+                                CLUTTER_PARAM_READABLE);
+  g_object_class_install_property (gobject_class, PROP_FILTER_SET, pspec);
 
    /**
    * ClutterModel::row-added:
@@ -419,6 +488,105 @@ clutter_model_check_type (GType gtype)
   return FALSE;
 }
 
+
+typedef struct {
+    gchar *name;
+    GType  type;
+} ColumnInfo;
+
+static gboolean
+clutter_model_parse_custom_node (ClutterScriptable *scriptable,
+                                 ClutterScript     *script,
+                                 GValue            *value,
+                                 const gchar       *name,
+                                 JsonNode          *node)
+{
+  GSList *columns = NULL;
+  GList *elements, *l;
+
+  if (strcmp (name, "columns") != 0)
+    return FALSE;
+
+  if (JSON_NODE_TYPE (node) != JSON_NODE_ARRAY)
+    return FALSE;
+
+  elements = json_array_get_elements (json_node_get_array (node));
+
+  for (l = elements; l != NULL; l = l->next)
+    {
+      JsonNode *child_node = l->data;
+      JsonArray *array = json_node_get_array (child_node);
+      ColumnInfo *cinfo;
+      const gchar *column_name;
+      const gchar *type_name;
+
+      if (JSON_NODE_TYPE (node) != JSON_NODE_ARRAY ||
+          json_array_get_length (array) != 2)
+        {
+          g_warning ("A column must be an array of "
+                     "[\"column-name\", \"GType-name\"] pairs");
+          return FALSE;
+        }
+
+      column_name = json_array_get_string_element (array, 0);
+      type_name = json_array_get_string_element (array, 1);
+
+      cinfo = g_slice_new0 (ColumnInfo);
+      cinfo->name = g_strdup (column_name);
+      cinfo->type = clutter_script_get_type_from_name (script, type_name);
+
+      columns = g_slist_prepend (columns, cinfo);
+    }
+
+  g_list_free (elements);
+
+  g_value_init (value, G_TYPE_POINTER);
+  g_value_set_pointer (value, g_slist_reverse (columns));
+
+  return TRUE;
+}
+
+static void
+clutter_model_set_custom_property (ClutterScriptable *scriptable,
+                                   ClutterScript     *script,
+                                   const gchar       *name,
+                                   const GValue      *value)
+{
+  if (strcmp (name, "columns") == 0)
+    {
+      ClutterModel *model = CLUTTER_MODEL (scriptable);
+      GSList *columns, *l;
+      guint n_columns;
+      gint i;
+
+      columns = g_value_get_pointer (value);
+      n_columns = g_slist_length (columns);
+
+      clutter_model_set_n_columns (model, n_columns, TRUE, TRUE);
+
+      for (i = 0, l = columns; l != NULL; l = l->next, i++)
+        {
+          ColumnInfo *cinfo = l->data;
+
+          clutter_model_set_column_name (model, i, cinfo->name);
+          clutter_model_set_column_type (model, i, cinfo->type);
+
+          g_free (cinfo->name);
+          g_slice_free (ColumnInfo, cinfo);
+        }
+
+      g_slist_free (columns);
+    }
+}
+
+
+static void
+clutter_scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_model_parse_custom_node;
+  iface->set_custom_property = clutter_model_set_custom_property;
+}
+
 /**
  * clutter_model_resort:
  * @model: a #ClutterModel
@@ -474,7 +642,7 @@ clutter_model_filter_row (ClutterModel *model,
     return TRUE;
 
   iter = clutter_model_get_iter_at_row (model, row);
-  if (!iter)
+  if (iter == NULL)
     return FALSE;
 
   res = priv->filter_func (model, iter, priv->filter_data);
@@ -536,10 +704,6 @@ clutter_model_set_n_columns (ClutterModel *model,
                              gboolean      set_names)
 {
   ClutterModelPrivate *priv = model->priv;
-  gboolean types_set, names_set;
-
-  types_set = (priv->column_types != NULL);
-  names_set = (priv->column_names != NULL);
 
   if (priv->n_columns > 0 && priv->n_columns != n_columns)
     return;
@@ -763,12 +927,10 @@ void
 clutter_model_append (ClutterModel *model,
                       ...)
 {
-  ClutterModelPrivate *priv;
   ClutterModelIter *iter;
   va_list args;
 
   g_return_if_fail (CLUTTER_IS_MODEL (model));
-  priv = model->priv;
 
   iter = CLUTTER_MODEL_GET_CLASS (model)->insert_row (model, -1);
   g_assert (CLUTTER_IS_MODEL_ITER (iter));
@@ -1057,12 +1219,10 @@ G_CONST_RETURN gchar *
 clutter_model_get_column_name (ClutterModel *model,
                                guint         column)
 {
-  ClutterModelPrivate *priv;
   ClutterModelClass *klass;
 
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), NULL);
 
-  priv = model->priv;
   if (column < 0 || column >= clutter_model_get_n_columns (model))
     {
       g_warning ("%s: Invalid column id value %d\n", G_STRLOC, column);
@@ -1091,12 +1251,10 @@ GType
 clutter_model_get_column_type (ClutterModel *model,
                                guint         column)
 {
-  ClutterModelPrivate *priv;
   ClutterModelClass *klass;
 
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), G_TYPE_INVALID);
 
-  priv = model->priv;
   if (column < 0 || column >= clutter_model_get_n_columns (model))
     {
       g_warning ("%s: Invalid column id value %d\n", G_STRLOC, column);
@@ -1117,6 +1275,10 @@ clutter_model_get_column_type (ClutterModel *model,
  *
  * Retrieves a #ClutterModelIter representing the row at the given index.
  *
+ * If a filter function has been set using clutter_model_set_filter()
+ * then the @model implementation will return the first non filtered
+ * row.
+ *
  * Return value: (transfer full): A new #ClutterModelIter, or %NULL if @row was
  *   out of bounds. When done using the iterator object, call g_object_unref()
  *   to deallocate its resources
@@ -1143,49 +1305,65 @@ clutter_model_get_iter_at_row (ClutterModel *model,
  * clutter_model_get_first_iter:
  * @model: a #ClutterModel
  *
- * Retrieves a #ClutterModelIter representing the first row in @model.
+ * Retrieves a #ClutterModelIter representing the first non-filtered
+ * row in @model.
  *
- * Return value: (transfer full): A new #ClutterModelIter. Call g_object_unref() when
- *   done using it
+ * Return value: (transfer full): A new #ClutterModelIter.
+ *   Call g_object_unref() when done using it
  *
  * Since: 0.6
  */
 ClutterModelIter *
 clutter_model_get_first_iter (ClutterModel *model)
 {
+  ClutterModelIter *retval;
+
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), NULL);
 
-  return clutter_model_get_iter_at_row (model, 0);
+  retval = clutter_model_get_iter_at_row (model, 0);
+  if (retval != NULL)
+    {
+      g_assert (clutter_model_filter_iter (model, retval) != FALSE);
+      g_assert (clutter_model_iter_get_row (retval) == 0);
+    }
+
+  return retval;
 }
 
 /**
  * clutter_model_get_last_iter:
  * @model: a #ClutterModel
  *
- * Retrieves a #ClutterModelIter representing the last row in @model.
+ * Retrieves a #ClutterModelIter representing the last non-filtered
+ * row in @model.
  *
- * Return value: (transfer full): A new #ClutterModelIter. Call g_object_unref() when
- *   done using it
+ * Return value: (transfer full): A new #ClutterModelIter.
+ *   Call g_object_unref() when done using it
  *
  * Since: 0.6
  */
 ClutterModelIter *
 clutter_model_get_last_iter (ClutterModel *model)
 {
+  ClutterModelIter *retval;
   guint length;
 
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), NULL);
 
   length = clutter_model_get_n_rows (model);
+  retval = clutter_model_get_iter_at_row (model, length - 1);
+  if (retval != NULL)
+    g_assert (clutter_model_filter_iter (model, retval) != FALSE);
 
-  return clutter_model_get_iter_at_row (model, length - 1);
+  return retval;
 }
 
 /**
  * clutter_model_get_n_rows:
  * @model: a #ClutterModel
  *
- * Retrieves the number of rows inside @model.
+ * Retrieves the number of rows inside @model, eventually taking
+ * into account any filtering function set using clutter_model_set_filter().
  *
  * Return value: The length of the @model. If there is a filter set, then
  *   the length of the filtered @model is returned.
@@ -1195,9 +1373,35 @@ clutter_model_get_last_iter (ClutterModel *model)
 guint
 clutter_model_get_n_rows (ClutterModel *model)
 {
+  ClutterModelClass *klass;
+  guint row_count;
+
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), 0);
 
-  return CLUTTER_MODEL_GET_CLASS (model)->get_n_rows (model);
+  klass = CLUTTER_MODEL_GET_CLASS (model);
+  if (klass->get_n_rows)
+    row_count = klass->get_n_rows (model);
+  else
+    {
+      ClutterModelIter *iter;
+
+      iter = clutter_model_get_first_iter (model);
+      if (iter == NULL)
+        return 0;
+
+      row_count = 0;
+      while (!clutter_model_iter_is_last (iter))
+        {
+          if (clutter_model_filter_iter (model, iter))
+            row_count += 1;
+
+          iter = clutter_model_iter_next (iter);
+        }
+
+      g_object_unref (iter);
+    }
+
+  return row_count;
 }
 
 
@@ -1313,6 +1517,9 @@ clutter_model_set_sort (ClutterModel         *model,
   ClutterModelPrivate *priv;
     
   g_return_if_fail (CLUTTER_IS_MODEL (model));
+  g_return_if_fail ((func != NULL && column >= 0) ||
+                    (func == NULL && column == -1));
+
   priv = model->priv;
 
   if (priv->sort_notify)
@@ -1356,6 +1563,26 @@ clutter_model_set_filter (ClutterModel           *model,
   priv->filter_notify = notify;
 
   g_signal_emit (model, model_signals[FILTER_CHANGED], 0);
+  g_object_notify (G_OBJECT (model), "filter-set");
+}
+
+/**
+ * clutter_model_get_filter_set:
+ * @model: a #ClutterModel
+ *
+ * Returns whether the @model has a filter in place, set
+ * using clutter_model_set_filter()
+ *
+ * Return value: %TRUE if a filter is set
+ *
+ * Since: 1.0
+ */
+gboolean
+clutter_model_get_filter_set (ClutterModel *model)
+{
+  g_return_val_if_fail (CLUTTER_IS_MODEL (model), FALSE);
+
+  return model->priv->filter_func != NULL;
 }
 
 /*
@@ -1418,12 +1645,20 @@ clutter_model_iter_real_get_row (ClutterModelIter *iter)
   return iter->priv->row;
 }
 
+/* private function */
+void
+clutter_model_iter_set_row (ClutterModelIter *iter,
+                            guint             row)
+{
+  iter->priv->row = row;
+}
+
 static void
 clutter_model_iter_get_value_unimplemented (ClutterModelIter *iter,
                                             guint             column,
                                             GValue           *value)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::get_value() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1434,7 +1669,7 @@ clutter_model_iter_set_value_unimplemented (ClutterModelIter *iter,
                                             guint             column,
                                             const GValue     *value)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::set_value() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1443,7 +1678,7 @@ clutter_model_iter_set_value_unimplemented (ClutterModelIter *iter,
 static gboolean
 clutter_model_iter_is_first_unimplemented (ClutterModelIter *iter)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::is_first() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1453,7 +1688,7 @@ clutter_model_iter_is_first_unimplemented (ClutterModelIter *iter)
 static gboolean
 clutter_model_iter_is_last_unimplemented (ClutterModelIter *iter)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::is_last() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1463,7 +1698,7 @@ clutter_model_iter_is_last_unimplemented (ClutterModelIter *iter)
 static ClutterModelIter *
 clutter_model_iter_next_unimplemented (ClutterModelIter *iter)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::next() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1473,7 +1708,7 @@ clutter_model_iter_next_unimplemented (ClutterModelIter *iter)
 static ClutterModelIter *
 clutter_model_iter_prev_unimplemented (ClutterModelIter *iter)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::prev() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1483,7 +1718,7 @@ clutter_model_iter_prev_unimplemented (ClutterModelIter *iter)
 static ClutterModelIter *
 clutter_model_iter_copy_unimplemented (ClutterModelIter *iter)
 {
-  g_warning ("%s: Iterator of type `%s' does not implement the "
+  g_warning ("%s: Iterator of type '%s' does not implement the "
              "ClutterModelIter::copy() virtual function",
              G_STRLOC,
              g_type_name (G_OBJECT_TYPE (iter)));
@@ -1539,7 +1774,8 @@ clutter_model_iter_set_property (GObject      *object,
 static void
 clutter_model_iter_class_init (ClutterModelIterClass *klass)
 {
-  GObjectClass        *gobject_class = G_OBJECT_CLASS (klass);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GParamSpec *pspec;
 
   gobject_class->get_property = clutter_model_iter_get_property;
   gobject_class->set_property = clutter_model_iter_set_property;
@@ -1563,28 +1799,26 @@ clutter_model_iter_class_init (ClutterModelIterClass *klass)
    *
    * Since: 0.6
    */
-  g_object_class_install_property (gobject_class,
-                                   ITER_PROP_MODEL,
-                                   g_param_spec_object ("model",
-                                                        "Model",
-                                                        "The model to which the iterator belongs to",
-                                                        CLUTTER_TYPE_MODEL,
-                                                        CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_object ("model",
+                               "Model",
+                               "The model to which the iterator belongs to",
+                               CLUTTER_TYPE_MODEL,
+                               CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, ITER_PROP_MODEL, pspec);
 
-   /**
+  /**
    * ClutterModelIter:row:
    *
    * The row number to which this iter points to.
    *
    * Since: 0.6
    */
-  g_object_class_install_property (gobject_class,
-                                   ITER_PROP_ROW,
-                                   g_param_spec_uint ("row",
-                                                      "Row",
-                                                      "The row to which the iterator points to",
-                                                      0, G_MAXUINT, 0,
-                                                      CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_uint ("row",
+                             "Row",
+                             "The row to which the iterator points to",
+                             0, G_MAXUINT, 0,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, ITER_PROP_ROW, pspec);
   
   g_type_class_add_private (gobject_class, sizeof (ClutterModelIterPrivate));
 }
