@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * ClutterTimeoutPool: pool of timeout functions using the same slice of
  *                     the GLib main loop
@@ -36,6 +36,7 @@
 
 #include "clutter-debug.h"
 #include "clutter-timeout-pool.h"
+#include "clutter-timeout-interval.h"
 
 typedef struct _ClutterTimeout  ClutterTimeout;
 typedef enum {
@@ -49,8 +50,7 @@ struct _ClutterTimeout
   ClutterTimeoutFlags flags;
   gint refcount;
 
-  guint interval;
-  guint last_time;
+  ClutterTimeoutInterval interval;
 
   GSourceFunc func;
   gpointer data;
@@ -94,7 +94,6 @@ clutter_timeout_sort (gconstpointer a,
 {
   const ClutterTimeout *t_a = a;
   const ClutterTimeout *t_b = b;
-  gint comparison;
 
   /* Keep 'ready' timeouts at the front */
   if (TIMEOUT_READY (t_a))
@@ -103,16 +102,8 @@ clutter_timeout_sort (gconstpointer a,
   if (TIMEOUT_READY (t_b))
     return 1;
 
-  /* Otherwise sort by expiration time */
-  comparison = (t_a->last_time + t_a->interval)
-    - (t_b->last_time + t_b->interval);
-  if (comparison < 0)
-    return -1;
-
-  if (comparison > 0)
-    return 1;
-
-  return 0;
+  return _clutter_timeout_interval_compare_expiration (&t_a->interval,
+                                                       &t_b->interval);
 }
 
 static gint
@@ -124,15 +115,17 @@ clutter_timeout_find_by_id (gconstpointer a,
   return t_a->id == GPOINTER_TO_UINT (b) ? 0 : 1;
 }
 
-static guint
-clutter_timeout_pool_get_ticks (ClutterTimeoutPool *pool)
+static ClutterTimeout *
+clutter_timeout_new (guint fps)
 {
-  GTimeVal time_now;
+  ClutterTimeout *timeout;
 
-  g_source_get_current_time ((GSource *) pool, &time_now);
-  
-  return (time_now.tv_sec - pool->start_time.tv_sec) * 1000
-    + (time_now.tv_usec - pool->start_time.tv_usec) / 1000;
+  timeout = g_slice_new0 (ClutterTimeout);
+  _clutter_timeout_interval_init (&timeout->interval, fps);
+  timeout->flags = CLUTTER_TIMEOUT_NONE;
+  timeout->refcount = 1;
+
+  return timeout;
 }
 
 static gboolean
@@ -140,66 +133,12 @@ clutter_timeout_prepare (ClutterTimeoutPool *pool,
                          ClutterTimeout     *timeout,
                          gint               *next_timeout)
 {
-  guint now = clutter_timeout_pool_get_ticks (pool);
+  GTimeVal now;
 
-  /* If time has gone backwards or the time since the last frame is
-     greater than the two frames worth then reset the time and do a
-     frame now */
-  if (timeout->last_time > now || now - timeout->last_time
-      > timeout->interval * 2)
-    {
-      timeout->last_time = now - timeout->interval;
-      if (next_timeout)
-	*next_timeout = 0;
-      return TRUE;
-    }
-  else if (now - timeout->last_time >= timeout->interval)
-    {
-      if (next_timeout)
-	*next_timeout = 0;
-      return TRUE;
-    }
-  else
-    {
-      if (next_timeout)
-	*next_timeout = timeout->interval + timeout->last_time - now;
-      return FALSE;
-    }
-}
+  g_source_get_current_time (&pool->source, &now);
 
-static gboolean
-clutter_timeout_dispatch (GSource        *source,
-                          ClutterTimeout *timeout)
-{
-  gboolean retval = FALSE;
-
-  if (G_UNLIKELY (!timeout->func))
-    {
-      g_warning ("Timeout dispatched without a callback.");
-      return FALSE;
-    }
-
-  if (timeout->func (timeout->data))
-    {
-      timeout->last_time += timeout->interval;
-
-      retval = TRUE;
-    }
-
-  return retval;
-}
-
-static ClutterTimeout *
-clutter_timeout_new (guint interval)
-{
-  ClutterTimeout *timeout;
-
-  timeout = g_slice_new0 (ClutterTimeout);
-  timeout->interval = interval;
-  timeout->flags = CLUTTER_TIMEOUT_NONE;
-  timeout->refcount = 1;
-
-  return timeout;
+  return _clutter_timeout_interval_prepare (&now, &timeout->interval,
+                                            next_timeout);
 }
 
 /* ref and unref are always called under the main Clutter lock, so there
@@ -270,7 +209,7 @@ static gboolean
 clutter_timeout_pool_check (GSource *source)
 {
   ClutterTimeoutPool *pool = (ClutterTimeoutPool *) source;
-  GList *l = pool->timeouts;
+  GList *l;
 
   clutter_threads_enter ();
 
@@ -350,7 +289,8 @@ clutter_timeout_pool_dispatch (GSource     *source,
       l->next = pool->dispatched_timeouts;
       pool->dispatched_timeouts = l;
 
-      if (!clutter_timeout_dispatch (source, timeout))
+      if (!_clutter_timeout_interval_dispatch (&timeout->interval,
+                                               timeout->func, timeout->data))
 	{
 	  /* The timeout may have already been removed, but nothing
            * can be added to the dispatched_timeout list except in this
@@ -419,10 +359,10 @@ clutter_timeout_pool_finalize (GSource *source)
  * always sorted, so that the extraction of the next timeout function is
  * a constant time operation.
  *
- * Inside Clutter, every #ClutterTimeline share the same timeout pool, unless
- * the CLUTTER_TIMELINE=no-pool environment variable is set.
- *
- * Return value: the newly created #ClutterTimeoutPool
+ * Return value: the newly created #ClutterTimeoutPool. The created pool
+ *   is owned by the GLib default context and will be automatically
+ *   destroyed when the context is destroyed. It is possible to force
+ *   the destruction of the timeout pool using g_source_destroy()
  *
  * Since: 0.4
  */
@@ -441,9 +381,12 @@ clutter_timeout_pool_new (gint priority)
     g_source_set_priority (source, priority);
 
   pool = (ClutterTimeoutPool *) source;
+
   g_get_current_time (&pool->start_time);
   pool->next_id = 1;
   pool->id = g_source_attach (source, NULL);
+
+  /* let the default GLib context manage the pool */
   g_source_unref (source);
 
   return pool;
@@ -452,7 +395,7 @@ clutter_timeout_pool_new (gint priority)
 /**
  * clutter_timeout_pool_add:
  * @pool: a #ClutterTimeoutPool
- * @interval: the time between calls to the function, in milliseconds
+ * @fps: the time between calls to the function, in frames per second
  * @func: function to call
  * @data: data to pass to the function, or %NULL
  * @notify: function to call when the timeout is removed, or %NULL
@@ -463,7 +406,7 @@ clutter_timeout_pool_new (gint priority)
  * won't be called again. If @notify is not %NULL, the @notify function
  * will be called. The first call to @func will be at the end of @interval.
  *
- * Since version 0.8 this will try to compensate for delays. For
+ * Since Clutter 0.8 this will try to compensate for delays. For
  * example, if @func takes half the interval time to execute then the
  * function will be called again half the interval time after it
  * finished. Before version 0.8 it would not fire until a full
@@ -479,7 +422,7 @@ clutter_timeout_pool_new (gint priority)
  */
 guint
 clutter_timeout_pool_add (ClutterTimeoutPool *pool,
-                          guint               interval,
+                          guint               fps,
                           GSourceFunc         func,
                           gpointer            data,
                           GDestroyNotify      notify)
@@ -487,11 +430,10 @@ clutter_timeout_pool_add (ClutterTimeoutPool *pool,
   ClutterTimeout *timeout;
   guint retval = 0;
 
-  timeout = clutter_timeout_new (interval);
+  timeout = clutter_timeout_new (fps);
 
   retval = timeout->id = pool->next_id++;
 
-  timeout->last_time = clutter_timeout_pool_get_ticks (pool);
   timeout->func = func;
   timeout->data = data;
   timeout->notify = notify;
@@ -530,7 +472,8 @@ clutter_timeout_pool_remove (ClutterTimeoutPool *pool,
 				    clutter_timeout_find_by_id)))
     {
       clutter_timeout_unref (l->data);
-      pool->dispatched_timeouts
-	= g_list_delete_link (pool->dispatched_timeouts, l);
+
+      pool->dispatched_timeouts =
+        g_list_delete_link (pool->dispatched_timeouts, l);
     }
 }
