@@ -20,9 +20,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -71,6 +71,7 @@
 
 #include "clutter-units.h"
 #include "clutter-private.h"
+#include "clutter-interval.h"
 
 #define DPI_FALLBACK    (96.0)
 
@@ -90,6 +91,12 @@ units_mm_to_pixels (gfloat mm)
   return mm * dpi / 25.4;
 }
 
+static gfloat
+units_cm_to_pixels (gfloat cm)
+{
+  return units_mm_to_pixels (cm * 10);
+}
+
 static gfloat
 units_pt_to_pixels (gfloat pt)
 {
@@ -144,12 +151,43 @@ void
 clutter_units_from_mm (ClutterUnits *units,
                        gfloat        mm)
 {
+  ClutterBackend *backend;
+
   g_return_if_fail (units != NULL);
 
+  backend = clutter_get_default_backend ();
+
   units->unit_type  = CLUTTER_UNIT_MM;
   units->value      = mm;
   units->pixels     = units_mm_to_pixels (mm);
   units->pixels_set = TRUE;
+  units->serial     = _clutter_backend_get_units_serial (backend);
+}
+
+/**
+ * clutter_units_from_cm:
+ * @units: a #ClutterUnits
+ * @cm: centimeters
+ *
+ * Stores a value in centimeters inside @units
+ *
+ * Since: 1.2
+ */
+void
+clutter_units_from_cm (ClutterUnits *units,
+                       gfloat        cm)
+{
+  ClutterBackend *backend;
+
+  g_return_if_fail (units != NULL);
+
+  backend = clutter_get_default_backend ();
+
+  units->unit_type  = CLUTTER_UNIT_CM;
+  units->value      = cm;
+  units->pixels     = units_cm_to_pixels (cm);
+  units->pixels_set = TRUE;
+  units->serial     = _clutter_backend_get_units_serial (backend);
 }
 
 /**
@@ -165,12 +203,17 @@ void
 clutter_units_from_pt (ClutterUnits *units,
                        gfloat        pt)
 {
+  ClutterBackend *backend;
+
   g_return_if_fail (units != NULL);
 
+  backend = clutter_get_default_backend ();
+
   units->unit_type  = CLUTTER_UNIT_POINT;
   units->value      = pt;
   units->pixels     = units_pt_to_pixels (pt);
   units->pixels_set = TRUE;
+  units->serial     = _clutter_backend_get_units_serial (backend);
 }
 
 /**
@@ -187,12 +230,17 @@ void
 clutter_units_from_em (ClutterUnits *units,
                        gfloat        em)
 {
+  ClutterBackend *backend;
+
   g_return_if_fail (units != NULL);
 
+  backend = clutter_get_default_backend ();
+
   units->unit_type  = CLUTTER_UNIT_EM;
   units->value      = em;
   units->pixels     = units_em_to_pixels (NULL, em);
   units->pixels_set = TRUE;
+  units->serial     = _clutter_backend_get_units_serial (backend);
 }
 
 /**
@@ -210,12 +258,17 @@ clutter_units_from_em_for_font (ClutterUnits *units,
                                 const gchar  *font_name,
                                 gfloat        em)
 {
+  ClutterBackend *backend;
+
   g_return_if_fail (units != NULL);
 
+  backend = clutter_get_default_backend ();
+
   units->unit_type  = CLUTTER_UNIT_EM;
   units->value      = em;
   units->pixels     = units_em_to_pixels (font_name, em);
   units->pixels_set = TRUE;
+  units->serial     = _clutter_backend_get_units_serial (backend);
 }
 
 /**
@@ -231,12 +284,17 @@ void
 clutter_units_from_pixels (ClutterUnits *units,
                            gint          px)
 {
+  ClutterBackend *backend;
+
   g_return_if_fail (units != NULL);
 
+  backend = clutter_get_default_backend ();
+
   units->unit_type  = CLUTTER_UNIT_PIXEL;
   units->value      = px;
   units->pixels     = px;
   units->pixels_set = TRUE;
+  units->serial     = _clutter_backend_get_units_serial (backend);
 }
 
 /**
@@ -326,14 +384,28 @@ clutter_units_free (ClutterUnits *units)
 gfloat
 clutter_units_to_pixels (ClutterUnits *units)
 {
+  ClutterBackend *backend;
+
   g_return_val_if_fail (units != NULL, 0.0);
 
+  /* if the backend settings changed we evict the cached value */
+  backend = clutter_get_default_backend ();
+  if (units->serial != _clutter_backend_get_units_serial (backend))
+    units->pixels_set = FALSE;
+
+  if (units->pixels_set)
+    return units->pixels;
+
   switch (units->unit_type)
     {
     case CLUTTER_UNIT_MM:
       units->pixels = units_mm_to_pixels (units->value);
       break;
 
+    case CLUTTER_UNIT_CM:
+      units->pixels = units_cm_to_pixels (units->value);
+      break;
+
     case CLUTTER_UNIT_POINT:
       units->pixels = units_pt_to_pixels (units->value);
       break;
@@ -348,6 +420,7 @@ clutter_units_to_pixels (ClutterUnits *units)
     }
 
   units->pixels_set = TRUE;
+  units->serial = _clutter_backend_get_units_serial (backend);
 
   return units->pixels;
 }
@@ -362,10 +435,14 @@ clutter_units_to_pixels (ClutterUnits *units)
  * A #ClutterUnits expressed in string should match:
  *
  * |[
- *   number: [0-9]
- *   unit_value: &lt;number&gt;+
- *   unit_name: px|pt|mm|em
- *   units: &lt;unit_value&gt; &lt;unit_name&gt;
+ *   units: wsp* unit-value wsp* unit-name? wsp*
+ *   unit-value: number
+ *   unit-name: 'px' | 'pt' | 'mm' | 'em' | 'cm'
+ *   number: digit+
+ *           | digit* sep digit+
+ *   sep: '.' | ','
+ *   digit: '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9'
+ *   wsp: (#0x20 | #0x9 | #0xA | #0xB | #0xC | #0xD)+
  * ]|
  *
  * For instance, these are valid strings:
@@ -375,6 +452,7 @@ clutter_units_to_pixels (ClutterUnits *units)
  *   5.1 em
  *   24 pt
  *   12.6 mm
+ *   .3 cm
  * ]|
  *
  * While these are not:
@@ -395,61 +473,85 @@ gboolean
 clutter_units_from_string (ClutterUnits *units,
                            const gchar  *str)
 {
+  ClutterBackend *backend;
   ClutterUnitType unit_type;
   gfloat value;
 
   g_return_val_if_fail (units != NULL, FALSE);
   g_return_val_if_fail (str != NULL, FALSE);
 
-  /* Ensure that the first character is a digit */
+  /* strip leading space */
   while (g_ascii_isspace (*str))
     str++;
 
   if (*str == '\0')
     return FALSE;
 
-  if (!g_ascii_isdigit (*str))
-    return FALSE;
-
   /* integer part */
   value = (gfloat) strtoul (str, (char **) &str, 10);
 
   if (*str == '.' || *str == ',')
     {
-      glong frac = 100000;
+      gfloat divisor = 0.1;
 
-      while (g_ascii_isdigit (*++str))
+      /* 5.cm is not a valid number */
+      if (!g_ascii_isdigit (*++str))
+        return FALSE;
+
+      while (g_ascii_isdigit (*str))
         {
-          frac += (*str - '0') * frac;
-          frac /= 10;
+          value += (*str - '0') * divisor;
+          divisor *= 0.1;
+          str++;
         }
-
-      value += (1.0f / (gfloat) frac);
     }
 
+  while (g_ascii_isspace (*str))
+    str++;
+
   /* assume pixels by default, if no unit is specified */
-  if (str == '\0')
+  if (*str == '\0')
     unit_type = CLUTTER_UNIT_PIXEL;
-  else
+  else if (strncmp (str, "em", 2) == 0)
     {
-      while (g_ascii_isspace (*str))
-        str++;
-
-      if (strncmp (str, "em", 2) == 0)
-        unit_type = CLUTTER_UNIT_EM;
-      else if (strncmp (str, "mm", 2) == 0)
-        unit_type = CLUTTER_UNIT_MM;
-      else if (strncmp (str, "pt", 2) == 0)
-        unit_type = CLUTTER_UNIT_POINT;
-      else if (strncmp (str, "px", 2) == 0)
-        unit_type = CLUTTER_UNIT_PIXEL;
-      else
-        return FALSE;
+      unit_type = CLUTTER_UNIT_EM;
+      str += 2;
     }
+  else if (strncmp (str, "mm", 2) == 0)
+    {
+      unit_type = CLUTTER_UNIT_MM;
+      str += 2;
+    }
+  else if (strncmp (str, "cm", 2) == 0)
+    {
+      unit_type = CLUTTER_UNIT_CM;
+      str += 2;
+    }
+  else if (strncmp (str, "pt", 2) == 0)
+    {
+      unit_type = CLUTTER_UNIT_POINT;
+      str += 2;
+    }
+  else if (strncmp (str, "px", 2) == 0)
+    {
+      unit_type = CLUTTER_UNIT_PIXEL;
+      str += 2;
+    }
+  else
+        return FALSE;
+
+  /* ensure the unit is only followed by white space */
+  while (g_ascii_isspace (*str))
+    str++;
+  if (*str != '\0')
+    return FALSE;
+
+  backend = clutter_get_default_backend ();
 
   units->unit_type = unit_type;
   units->value = value;
   units->pixels_set = FALSE;
+  units->serial = _clutter_backend_get_units_serial (backend);
 
   return TRUE;
 }
@@ -462,6 +564,9 @@ clutter_unit_type_name (ClutterUnitType unit_type)
     case CLUTTER_UNIT_MM:
       return "mm";
 
+    case CLUTTER_UNIT_CM:
+      return "cm";
+
     case CLUTTER_UNIT_POINT:
       return "pt";
 
@@ -487,7 +592,7 @@ clutter_unit_type_name (ClutterUnitType unit_type)
  * examples of output
  *
  * <note>Fractional values are truncated to the second decimal
- * position for em and mm, and to the first decimal position for
+ * position for em, mm and cm, and to the first decimal position for
  * typographic points. Pixels are integers.</note>
  *
  * Return value: a newly allocated string containing the encoded
@@ -499,30 +604,37 @@ gchar *
 clutter_units_to_string (const ClutterUnits *units)
 {
   const gchar *unit_name = NULL;
-  gchar *fmt = NULL;
+  const gchar *fmt = NULL;
+  gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
 
   g_return_val_if_fail (units != NULL, NULL);
 
   switch (units->unit_type)
     {
+    /* special case: there is no such thing as "half a pixel", so
+     * we round up to the nearest integer using C default
+     */
+    case CLUTTER_UNIT_PIXEL:
+      return g_strdup_printf ("%d px", (int) units->value);
+
     case CLUTTER_UNIT_MM:
       unit_name = "mm";
-      fmt = g_strdup_printf ("%.2f", units->value);
+      fmt = "%.2f";
+      break;
+
+    case CLUTTER_UNIT_CM:
+      unit_name = "cm";
+      fmt = "%.2f";
       break;
 
     case CLUTTER_UNIT_POINT:
       unit_name = "pt";
-      fmt = g_strdup_printf ("%.1f", units->value);
+      fmt = "%.1f";
       break;
 
     case CLUTTER_UNIT_EM:
       unit_name = "em";
-      fmt = g_strdup_printf ("%.2f", units->value);
-      break;
-
-    case CLUTTER_UNIT_PIXEL:
-      unit_name = "px";
-      fmt = g_strdup_printf ("%d", (int) units->value);
+      fmt = "%.2f";
       break;
 
     default:
@@ -530,7 +642,34 @@ clutter_units_to_string (const ClutterUnits *units)
       break;
     }
 
-  return g_strconcat (fmt, " ", unit_name, NULL);
+  g_ascii_formatd (buf, G_ASCII_DTOSTR_BUF_SIZE, fmt, units->value);
+
+  return g_strconcat (buf, " ", unit_name, NULL);
+}
+
+/*
+ * ClutterInterval integration
+ */
+
+static gboolean
+clutter_units_progress (const GValue *a,
+                        const GValue *b,
+                        gdouble       progress,
+                        GValue       *retval)
+{
+  ClutterUnits *a_units = (ClutterUnits *) clutter_value_get_units (a);
+  ClutterUnits *b_units = (ClutterUnits *) clutter_value_get_units (b);
+  ClutterUnits  res;
+  gfloat a_px, b_px, value;
+
+  a_px = clutter_units_to_pixels (a_units);
+  b_px = clutter_units_to_pixels (b_units);
+  value = progress * (b_px - a_px) + a_px;
+
+  clutter_units_from_pixels (&res, value);
+  clutter_value_set_units (retval, &res);
+
+  return TRUE;
 }
 
 /*
@@ -618,6 +757,9 @@ clutter_units_get_type (void)
       g_value_register_transform_func (G_TYPE_STRING, clutter_units_type,
                                        clutter_value_transform_string_units);
 
+      clutter_interval_register_progress_func (clutter_units_type,
+                                               clutter_units_progress);
+
       g_once_init_leave (&clutter_units_type__volatile, clutter_units_type);
     }
 
@@ -653,7 +795,7 @@ clutter_value_set_units (GValue             *value,
  * Since: 0.8
  */
 G_CONST_RETURN ClutterUnits *
-clutter_value_get_unit (const GValue *value)
+clutter_value_get_units (const GValue *value)
 {
   g_return_val_if_fail (CLUTTER_VALUE_HOLDS_UNITS (value), NULL);
 
@@ -703,7 +845,7 @@ param_units_validate (GParamSpec *pspec,
       g_warning ("The units value of '%s' does not have the same unit "
                  "type as declared by the ClutterParamSpecUnits of '%s'",
                  str,
-                 clutter_unit_type_name (otype));
+                 clutter_unit_type_name (uspec->default_type));
 
       g_free (str);
 
