@@ -18,49 +18,70 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
-#ifndef _HAVE_CLUTTER_H
-#define _HAVE_CLUTTER_H
+#ifndef __CLUTTER_H__
+#define __CLUTTER_H__
+
+#define __CLUTTER_H_INSIDE__
+
+#include "clutter-deprecated.h"
 
 #include "clutter-actor.h"
 #include "clutter-alpha.h"
+#include "clutter-animatable.h"
+#include "clutter-animation.h"
+#include "clutter-animator.h"
 #include "clutter-backend.h"
-#include "clutter-color.h"
-#include "clutter-container.h"
-#include "clutter-behaviour.h"
-#include "clutter-behaviour-bspline.h"
 #include "clutter-behaviour-depth.h"
 #include "clutter-behaviour-ellipse.h"
+#include "clutter-behaviour.h"
 #include "clutter-behaviour-opacity.h"
 #include "clutter-behaviour-path.h"
 #include "clutter-behaviour-rotate.h"
 #include "clutter-behaviour-scale.h"
-#include "clutter-clone-texture.h"
-#include "clutter-deprecated.h"
-#include "clutter-effect.h"
-#include "clutter-entry.h"
+#include "clutter-binding-pool.h"
+#include "clutter-bin-layout.h"
+#include "clutter-box.h"
+#include "clutter-box-layout.h"
+#include "clutter-cairo-texture.h"
+#include "clutter-child-meta.h"
+#include "clutter-clone.h"
+#include "clutter-color.h"
+#include "clutter-container.h"
+#include "clutter-device-manager.h"
 #include "clutter-event.h"
 #include "clutter-feature.h"
-#include "clutter-rectangle.h"
+#include "clutter-fixed-layout.h"
+#include "clutter-flow-layout.h"
+#include "clutter-frame-source.h"
 #include "clutter-group.h"
+#include "clutter-input-device.h"
+#include "clutter-interval.h"
 #include "clutter-keysyms.h" 
-#include "clutter-label.h"
+#include "clutter-layout-manager.h"
+#include "clutter-layout-meta.h"
 #include "clutter-list-model.h"
 #include "clutter-main.h"
 #include "clutter-media.h"
 #include "clutter-model.h"
-#include "clutter-stage.h"
-#include "clutter-texture.h"
-#include "clutter-frame-source.h"
-#include "clutter-timeout-pool.h"
-#include "clutter-timeline.h"
+#include "clutter-path.h"
+#include "clutter-rectangle.h"
 #include "clutter-score.h"
-#include "clutter-script.h"
 #include "clutter-scriptable.h"
+#include "clutter-script.h"
+#include "clutter-shader.h"
+#include "clutter-shader-types.h"
+#include "clutter-stage.h"
+#include "clutter-stage-manager.h"
+#include "clutter-stage-window.h"
+#include "clutter-texture.h"
+#include "clutter-text.h"
+#include "clutter-timeline.h"
+#include "clutter-timeout-pool.h"
 #include "clutter-types.h"
 #include "clutter-units.h"
 #include "clutter-util.h"
@@ -68,4 +89,6 @@
 
 #include "clutter-enum-types.h"
 
-#endif
+#undef __CLUTTER_H_INSIDE__
+
+#endif /* __CLUTTER_H__ */
