@@ -3,8 +3,9 @@
 
 #include <glib-object.h>
 #include <clutter/clutter-stage.h>
-#include <GLES/gl.h>
-#include <GLES/egl.h>
+
+#include "clutter-egl-headers.h"
+#include "clutter-backend-egl.h"
 
 #define CLUTTER_TYPE_STAGE_EGL                  (clutter_stage_egl_get_type ())
 #define CLUTTER_STAGE_EGL(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLUTTER_TYPE_STAGE_EGL, ClutterStageEGL))
@@ -18,19 +19,18 @@ typedef struct _ClutterStageEGLClass    ClutterStageEGLClass;
 
 struct _ClutterStageEGL
 {
-  ClutterStage parent_instance;
+  ClutterActor parent_instance;
 
-  /* from the backend */
-  gint         surface_width;
-  gint         surface_height;
+  /* the stage wrapper */
+  ClutterStage      *wrapper;
 
-  EGLSurface   egl_surface;
-  EGLContext   egl_context;
+  /* back pointer to the backend */
+  ClutterBackendEGL *backend;
 };
 
 struct _ClutterStageEGLClass
 {
-  ClutterStageClass parent_class;
+  ClutterActorClass parent_class;
 };
 
 GType clutter_stage_egl_get_type (void) G_GNUC_CONST;
