@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __CLUTTER_BACKEND_EGL_H__
@@ -28,8 +28,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-#include <GLES/gl.h>
-#include <GLES/egl.h>
+#include "clutter-egl-headers.h"
 
 #include "../x11/clutter-backend-x11.h"
 #include "clutter-eglx.h"
@@ -52,7 +51,13 @@ struct _ClutterBackendEGL
 
   /* EGL Specific */
   EGLDisplay edpy;
-  gint       egl_version_major, egl_version_minor;
+  EGLContext egl_context;
+  EGLConfig  egl_config;
+  Window     dummy_xwin;
+  EGLSurface dummy_surface;
+
+  gint egl_version_major;
+  gint egl_version_minor;
 
 };
 
