@@ -8,8 +8,7 @@
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
 
-#include <GLES/gl.h>
-#include <GLES/egl.h>
+#include "clutter-egl-headers.h"
 
 #include "../x11/clutter-stage-x11.h"
 
@@ -27,8 +26,7 @@ struct _ClutterStageEGL
 {
   ClutterStageX11 parent_instance;
 
-  EGLSurface   egl_surface;
-  EGLContext   egl_context;
+  EGLSurface egl_surface;
 };
 
 struct _ClutterStageEGLClass
@@ -38,4 +36,7 @@ struct _ClutterStageEGLClass
 
 GType clutter_stage_egl_get_type (void) G_GNUC_CONST;
 
+void  clutter_stage_egl_redraw   (ClutterStageEGL *stage_egl,
+                                  ClutterStage    *stage);
+
 #endif /* __CLUTTER_STAGE_EGL_H__ */
