@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,9 +28,11 @@
 #include "clutter-backend-win32.h"
 #include "clutter-stage-win32.h"
 #include "clutter-win32.h"
+#include "clutter-device-manager-win32.h"
 
 #include "../clutter-event.h"
 #include "../clutter-main.h"
+#include "../clutter-input-device.h"
 #include "../clutter-debug.h"
 #include "../clutter-private.h"
 #include "../clutter-version.h"
@@ -47,6 +49,8 @@ static ClutterBackendWin32 *backend_singleton = NULL;
 
 static gchar *clutter_vblank_name = NULL;
 
+static HINSTANCE clutter_hinst = NULL;
+
 gboolean
 clutter_backend_win32_pre_parse (ClutterBackend  *backend,
 				 GError         **error)
@@ -62,11 +66,30 @@ clutter_backend_win32_pre_parse (ClutterBackend  *backend,
 static void
 clutter_backend_win32_init_events (ClutterBackend *backend)
 {
+  ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
+
   CLUTTER_NOTE (EVENT, "initialising the event loop");
 
+  backend_win32->device_manager =
+    g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_WIN32,
+                  "backend", backend_win32,
+                  NULL);
+
   _clutter_backend_win32_events_init (backend);
 }
 
+HCURSOR
+_clutter_backend_win32_get_invisible_cursor (ClutterBackend *backend)
+{
+  ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
+
+  if (backend_win32->invisible_cursor == NULL)
+    backend_win32->invisible_cursor =
+      LoadCursor (clutter_hinst, MAKEINTRESOURCE (42));
+
+  return backend_win32->invisible_cursor;
+}
+
 static const GOptionEntry entries[] =
   {
     {
@@ -119,10 +142,23 @@ clutter_backend_win32_dispose (GObject *gobject)
 
   if (backend_win32->gl_context)
     {
+      wglMakeCurrent (NULL, NULL);
       wglDeleteContext (backend_win32->gl_context);
       backend_win32->gl_context = NULL;
     }
 
+  if (backend_win32->dummy_dc)
+    {
+      ReleaseDC (backend_win32->dummy_hwnd, backend_win32->dummy_dc);
+      backend_win32->dummy_dc = NULL;
+    }
+
+  if (backend_win32->dummy_hwnd)
+    {
+      DestroyWindow (backend_win32->dummy_hwnd);
+      backend_win32->dummy_hwnd = NULL;
+    }
+
   G_OBJECT_CLASS (clutter_backend_win32_parent_class)->dispose (gobject);
 }
 
@@ -164,21 +200,20 @@ clutter_backend_win32_get_features (ClutterBackend *backend)
   SwapIntervalProc     swap_interval;
   ClutterBackendWin32 *backend_win32;
 
-  /* FIXME: we really need to check if gl context is set */
-
-  extensions = (const gchar *) glGetString (GL_EXTENSIONS);
-
   /* this will make sure that the GL context exists and is bound to a
      drawable */
   backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
   g_return_val_if_fail (backend_win32->gl_context != NULL, 0);
   g_return_val_if_fail (wglGetCurrentDC () != NULL, 0);
 
-  CLUTTER_NOTE (BACKEND, "Checking features\n"
-                "GL_VENDOR: %s\n"
-                "GL_RENDERER: %s\n"
-                "GL_VERSION: %s\n"
-                "GL_EXTENSIONS: %s\n",
+  extensions = (const gchar *) glGetString (GL_EXTENSIONS);
+
+  CLUTTER_NOTE (BACKEND,
+                "Checking features\n"
+                "  GL_VENDOR: %s\n"
+                "  GL_RENDERER: %s\n"
+                "  GL_VERSION: %s\n"
+                "  GL_EXTENSIONS: %s\n",
                 glGetString (GL_VENDOR),
                 glGetString (GL_RENDERER),
                 glGetString (GL_VERSION),
@@ -193,7 +228,7 @@ clutter_backend_win32_get_features (ClutterBackend *backend)
      for the swap control extension */
   if (getenv ("__GL_SYNC_TO_VBLANK") || check_vblank_env ("default"))
     CLUTTER_NOTE (BACKEND, "vblank sync: left at default at user request");
-  else if (cogl_check_extension ("WGL_EXT_swap_control", extensions)
+  else if (_cogl_check_extension ("WGL_EXT_swap_control", extensions)
 	   && (swap_interval = (SwapIntervalProc)
 	       cogl_get_proc_address ("wglSwapIntervalEXT")))
     {
@@ -224,14 +259,175 @@ clutter_backend_win32_get_features (ClutterBackend *backend)
   else
     CLUTTER_NOTE (BACKEND, "no use-able vblank mechanism found");
 
+  CLUTTER_NOTE (BACKEND, "backend features checked");
+
   return flags;
 }
 
+static ATOM
+clutter_backend_win32_get_dummy_window_class ()
+{
+  static ATOM klass = 0;
+
+  if (klass == 0)
+    {
+      WNDCLASSW wndclass;
+      memset (&wndclass, 0, sizeof (wndclass));
+      wndclass.lpfnWndProc = DefWindowProc;
+      wndclass.hInstance = GetModuleHandleW (NULL);
+      wndclass.lpszClassName = L"ClutterBackendWin32DummyWindow";
+      klass = RegisterClassW (&wndclass);
+    }
+
+  return klass;
+}
+
+static gboolean
+clutter_backend_win32_pixel_format_is_better (const PIXELFORMATDESCRIPTOR *pfa,
+                                              const PIXELFORMATDESCRIPTOR *pfb)
+{
+  /* Always prefer a format with a stencil buffer */
+  if (pfa->cStencilBits == 0)
+    {
+      if (pfb->cStencilBits > 0)
+        return TRUE;
+    }
+  else if (pfb->cStencilBits == 0)
+    return FALSE;
+
+  /* Prefer a bigger color buffer */
+  if (pfb->cColorBits > pfa->cColorBits)
+    return TRUE;
+  else if (pfb->cColorBits < pfa->cColorBits)
+    return FALSE;
+
+  /* Prefer a bigger depth buffer */
+  return pfb->cDepthBits > pfa->cDepthBits;
+}
+
+static int
+clutter_backend_win32_choose_pixel_format (HDC dc, PIXELFORMATDESCRIPTOR *pfd)
+{
+  int i, num_formats, best_pf = 0;
+  PIXELFORMATDESCRIPTOR best_pfd;
+
+  num_formats = DescribePixelFormat (dc, 0, sizeof (best_pfd), NULL);
+
+  for (i = 1; i <= num_formats; i++)
+    {
+      memset (pfd, 0, sizeof (*pfd));
+
+      if (DescribePixelFormat (dc, i, sizeof (best_pfd), pfd)
+          /* Check whether this format is useable by Clutter */
+          && ((pfd->dwFlags & (PFD_SUPPORT_OPENGL
+                               | PFD_DRAW_TO_WINDOW
+                               | PFD_DOUBLEBUFFER
+                               | PFD_GENERIC_FORMAT))
+              == (PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER | PFD_DRAW_TO_WINDOW))
+          && pfd->iPixelType == PFD_TYPE_RGBA
+          && pfd->cColorBits >= 16 && pfd->cColorBits <= 32
+          && pfd->cDepthBits >= 16 && pfd->cDepthBits <= 32
+          /* Check whether this is a better format than one we've
+             already found */
+          && (best_pf == 0
+              || clutter_backend_win32_pixel_format_is_better (&best_pfd, pfd)))
+        {
+          best_pf = i;
+          best_pfd = *pfd;
+        }
+    }
+
+  *pfd = best_pfd;
+
+  return best_pf;
+}
+
+static gboolean
+clutter_backend_win32_create_context (ClutterBackend  *backend,
+                                      GError         **error)
+{
+  ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
+
+  /* COGL assumes that there is always a GL context selected; in order
+   * to make sure that a WGL context exists and is made current, we
+   * use a small dummy window that never gets shown to which we can
+   * always fall back if no stage is available
+   */
+  if (backend_win32->dummy_hwnd == NULL)
+    {
+      ATOM window_class = clutter_backend_win32_get_dummy_window_class ();
+
+      if (window_class == 0)
+        {
+          g_critical ("Unable to register window class");
+          return FALSE;
+        }
+
+      backend_win32->dummy_hwnd =
+        CreateWindowW ((LPWSTR) MAKEINTATOM (window_class),
+                       L".",
+                       WS_OVERLAPPEDWINDOW,
+                       CW_USEDEFAULT,
+                       CW_USEDEFAULT,
+                       1, 1,
+                       NULL, NULL,
+                       GetModuleHandle (NULL),
+                       NULL);
+
+      if (backend_win32->dummy_hwnd == NULL)
+        {
+          g_critical ("Unable to create dummy window");
+          return FALSE;
+        }
+    }
+
+  if (backend_win32->dummy_dc == NULL)
+    {
+      PIXELFORMATDESCRIPTOR pfd;
+      int pf;
+
+      backend_win32->dummy_dc = GetDC (backend_win32->dummy_hwnd);
+
+      pf = clutter_backend_win32_choose_pixel_format (backend_win32->dummy_dc,
+                                                      &pfd);
+
+      if (pf == 0 || !SetPixelFormat (backend_win32->dummy_dc, pf, &pfd))
+        {
+          g_critical ("Unable to find suitable GL pixel format");
+          ReleaseDC (backend_win32->dummy_hwnd, backend_win32->dummy_dc);
+          backend_win32->dummy_dc = NULL;
+          return FALSE;
+        }
+    }
+
+  if (backend_win32->gl_context == NULL)
+    {
+      backend_win32->gl_context = wglCreateContext (backend_win32->dummy_dc);
+
+      if (backend_win32->gl_context == NULL)
+        {
+          g_critical ("Unable to create suitable GL context");
+          return FALSE;
+        }
+    }
+
+  CLUTTER_NOTE (BACKEND, "Selecting dummy 0x%x for the WGL context",
+                (unsigned int) backend_win32->dummy_hwnd);
+
+  wglMakeCurrent (backend_win32->dummy_dc, backend_win32->gl_context);
+
+  return TRUE;
+}
+
 static void
 clutter_backend_win32_ensure_context (ClutterBackend *backend, 
 				      ClutterStage   *stage)
 {
-  if (stage == NULL)
+  ClutterStageWindow  *impl;
+
+  if (stage == NULL ||
+      (CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_IN_DESTRUCTION) ||
+      ((impl = _clutter_stage_get_window (stage)) == NULL))
     {
       CLUTTER_NOTE (MULTISTAGE, "Clearing all context");
 
@@ -241,9 +437,7 @@ clutter_backend_win32_ensure_context (ClutterBackend *backend,
     {
       ClutterBackendWin32 *backend_win32;
       ClutterStageWin32   *stage_win32;
-      ClutterStageWindow  *impl;
 
-      impl = _clutter_stage_get_window (stage);
       g_return_if_fail (impl != NULL);
       
       CLUTTER_NOTE (MULTISTAGE, "Setting context for stage of type %s [%p]",
@@ -255,28 +449,32 @@ clutter_backend_win32_ensure_context (ClutterBackend *backend,
       
       /* no GL context to set */
       if (backend_win32->gl_context == NULL)
-	return;
+        return;
 
       /* we might get here inside the final dispose cycle, so we
        * need to handle this gracefully
        */
       if (stage_win32->client_dc == NULL)
-	{
-	  CLUTTER_NOTE (MULTISTAGE,
-			"Received a stale stage, clearing all context");
-  	 
-	  wglMakeCurrent (NULL, NULL);
-	}
+        {
+          CLUTTER_NOTE (MULTISTAGE,
+                        "Received a stale stage, clearing all context");
+
+          if (backend_win32->dummy_dc != NULL)
+            wglMakeCurrent (backend_win32->dummy_dc,
+                            backend_win32->gl_context);
+          else
+            wglMakeCurrent (NULL, NULL);
+        }
       else
-	{
-	  CLUTTER_NOTE (BACKEND,
+        {
+          CLUTTER_NOTE (BACKEND,
 			"MakeCurrent window %p (%s), context %p",
 			stage_win32->hwnd,
 			stage_win32->is_foreign_win ? "foreign" : "native",
 			backend_win32->gl_context);
-	  wglMakeCurrent (stage_win32->client_dc,
+          wglMakeCurrent (stage_win32->client_dc,
 			  backend_win32->gl_context);
-	}
+        }
     }
 }
 
@@ -297,19 +495,20 @@ clutter_backend_win32_redraw (ClutterBackend *backend,
 
   /* this will cause the stage implementation to be painted */
   clutter_actor_paint (CLUTTER_ACTOR (stage));
+  cogl_flush ();
 
   if (stage_win32->client_dc)
     SwapBuffers (stage_win32->client_dc);
 }
 
-static ClutterActor *
+static ClutterStageWindow *
 clutter_backend_win32_create_stage (ClutterBackend  *backend,
 				    ClutterStage    *wrapper,
 				    GError         **error)
 {
   ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
   ClutterStageWin32 *stage_win32;
-  ClutterActor *stage;
+  ClutterStageWindow *stage;
 
   CLUTTER_NOTE (BACKEND, "Creating stage of type '%s'",
 		g_type_name (CLUTTER_STAGE_TYPE));
@@ -324,6 +523,22 @@ clutter_backend_win32_create_stage (ClutterBackend  *backend,
   return stage;
 }
 
+static ClutterDeviceManager *
+clutter_backend_win32_get_device_manager (ClutterBackend *backend)
+{
+  ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
+
+  if (G_UNLIKELY (backend_win32->device_manager == NULL))
+    {
+      backend_win32->device_manager =
+        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_WIN32,
+                      "backend", backend_win32,
+                      NULL);
+    }
+
+  return backend_win32->device_manager;
+}
+
 static void
 clutter_backend_win32_class_init (ClutterBackendWin32Class *klass)
 {
@@ -340,7 +555,9 @@ clutter_backend_win32_class_init (ClutterBackendWin32Class *klass)
   backend_class->add_options      = clutter_backend_win32_add_options;
   backend_class->get_features     = clutter_backend_win32_get_features;
   backend_class->redraw           = clutter_backend_win32_redraw;
+  backend_class->create_context   = clutter_backend_win32_create_context;
   backend_class->ensure_context   = clutter_backend_win32_ensure_context;
+  backend_class->get_device_manager = clutter_backend_win32_get_device_manager;
 }
 
 static void
@@ -350,6 +567,7 @@ clutter_backend_win32_init (ClutterBackendWin32 *backend_win32)
 
   backend_win32->gl_context         = NULL;
   backend_win32->no_event_retrieval = FALSE;
+  backend_win32->invisible_cursor   = NULL;
 
   /* FIXME: get from GetSystemMetric? */
   clutter_backend_set_double_click_time (backend, 250);
@@ -367,3 +585,13 @@ _clutter_backend_impl_get_type (void)
 {
   return clutter_backend_win32_get_type ();
 }
+
+BOOL WINAPI
+DllMain (HINSTANCE hinst, DWORD reason, LPVOID reserved)
+{
+  if (reason == DLL_PROCESS_ATTACH)
+    /* Store the module handle so that we can use it to load resources */
+    clutter_hinst = hinst;
+
+  return TRUE;
+}
