@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __CLUTTER_STAGE_WIN32_H__
@@ -52,6 +52,8 @@ struct _ClutterStageWin32
   RECT         fullscreen_rect;
   gboolean     is_foreign_win;
   gboolean     tracking_mouse;
+  wchar_t     *wtitle;
+  gboolean        is_cursor_visible;
 
   ClutterBackendWin32 *backend;
   ClutterStageState   state;
@@ -78,6 +80,8 @@ LRESULT CALLBACK _clutter_stage_win32_window_proc (HWND hwnd,
 void _clutter_stage_win32_get_min_max_info (ClutterStageWin32 *stage_win32,
 					    MINMAXINFO *min_max_info);
 
+void _clutter_stage_win32_update_cursor (ClutterStageWin32 *stage_win32);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_STAGE_H__ */
